@@ -237,7 +237,7 @@ class Document(object):
             return False
         return True 
     
-    def commit(self, db):
+    def commit(self, db, safe=True):
         ''' Save this object to the database and set the ``_id`` field of this
             document to the returned id.
             
@@ -246,7 +246,7 @@ class Document(object):
         collection = db[self.get_collection_name()]
         for index in self.get_indexes():
             index.ensure(collection)
-        id = collection.save(self.wrap())
+        id = collection.save(self.wrap(), safe=safe)
         self.mongo_id = id
     
     def wrap(self):
@@ -387,7 +387,7 @@ class DocumentField(Field):
         ''' Returns a dict of the operations needed to update this object.  
             See :func:`Document.get_dirty_ops` for more details.'''
         try:
-            document = getattr(instance, self.name)
+            document = getattr(instance, self._name)
         except AttributeError:
             return {}
         if len(document._dirty) == 0:
@@ -399,7 +399,7 @@ class DocumentField(Field):
         for op, values in ops.iteritems():
             ret[op] = {}
             for key, value in values.iteritems():
-                name = '%s.%s' % (self.name, key)
+                name = '%s.%s' % (self._name, key)
                 ret[op][name] = value
         return ret
     
@@ -517,4 +517,4 @@ class Index(object):
         collection.ensure_index(self.components, unique=self.__unique, 
             drop_dups=self.__drop_dups)
         return self
-        
\ No newline at end of file
+        
