@@ -50,12 +50,14 @@ from itertools import chain
 
 class Session(object):
 
-    def __init__(self, database):
+    def __init__(self, database, safe=False):
         '''
         Create a session connecting to `database`.  
         
         :param database: the database to connect to.  Should be an instance of \
             :class:`pymongo.database.Database`
+        :param safe: Whether the "safe" option should be used on mongo writes, \
+            blocking to make sure there are no errors.
         
         **Fields**:
             * db: the underlying pymongo database object
@@ -64,21 +66,23 @@ class Session(object):
         '''
         self.db = database
         self.queue = []
-        self.safe = False
+        self.safe = safe
     
     @classmethod
-    def connect(self, database, *args, **kwds):
+    def connect(self, database, safe=False, *args, **kwds):
         ''' `connect` is a thin wrapper around __init__ which creates the 
             database connection that the session will use.
             
             :param database: the database name to use.  Should be an instance of \
                     :class:`basestring`
+            :param safe: The value for the "safe" parameter of the Session \ 
+                init function
             :param args: arguments for :class:`pymongo.connection.Connection`
             :param kwds: keyword arguments for :class:`pymongo.connection.Connection`
         '''
         conn = Connection(*args, **kwds)
         db = conn[database]
-        return Session(db)
+        return Session(db, safe=safe)
     
     def end(self):
         ''' End the session.  Flush all pending operations and ending the 
@@ -86,13 +90,15 @@ class Session(object):
         self.flush()
         self.db.connection.end_request()
     
-    def insert(self, item):
+    def insert(self, item, safe=None):
         ''' Insert an item into the queue and flushes.  Later this function should be smart and delay 
             insertion until the _id field is actually accessed'''
+        if safe is None:
+            safe = self.safe
         self.queue.append(item)
-        self.flush()
+        self.flush(safe=safe)
     
-    def update(self, item, id_expression=None, upsert=False, update_ops={}, **kwargs):
+    def update(self, item, id_expression=None, upsert=False, update_ops={}, safe=None, **kwargs):
         ''' Update an item in the database.  Uses the on_update keyword to each
             field to decide which operations to do, or.  
             
@@ -122,7 +128,7 @@ class Session(object):
             db_key = Query(type(item), self).filter(id_expression).query
         else:
             db_key = {'_id' : item.mongo_id}
-        
+
         dirty_ops = item.get_dirty_ops(with_required=upsert)
         for key, op in chain(update_ops.items(), kwargs.items()):
             key = str(key)
@@ -133,9 +139,10 @@ class Session(object):
                 del dirty_ops[current_op][key]
                 if len(dirty_ops[current_op]) == 0:
                     del dirty_ops[current_op]
-        
-        self.flush()
-        self.db[item.get_collection_name()].update(db_key, dirty_ops, upsert=upsert)
+        if safe is None:
+            safe = self.safe
+        self.flush(safe=safe)
+        self.db[item.get_collection_name()].update(db_key, dirty_ops, upsert=upsert, safe=safe)
         
     
     def query(self, type):
@@ -172,7 +179,7 @@ class Session(object):
             cursor.limit(query.get_limit())
         if query.get_skip() != None:
             cursor.skip(query.get_skip())
-        return QueryResult(cursor, query.type, fields=query.get_fields())
+        return QueryResult(cursor, query.type, raw_output=query._raw_output, fields=query.get_fields())
     
     def remove_query(self, type):
         ''' Begin a remove query on the database's collection for `type`.
@@ -192,10 +199,13 @@ class Session(object):
                 to the session's ``safe`` value.
         '''
         self.flush()
-        if safe == None:
+        if safe is None:
             safe = self.safe
         if not obj.has_id():
             return None
+        collection = self.db[obj.get_collection_name()]
+        for index in obj.get_indexes():
+            index.ensure(collection)
         return self.db[obj.get_collection_name()].remove(obj.mongo_id, safe=safe)
     
     def execute_remove(self, remove):
@@ -205,12 +215,17 @@ class Session(object):
         safe = self.safe
         if remove.safe != None:
             safe = remove.safe
+        
+        collection = self.db[remove.type.get_collection_name()]
+        for index in remove.type.get_indexes():
+            index.ensure(collection)
+
         return self.db[remove.type.get_collection_name()].remove(remove.query, safe=safe)
     
     def execute_update(self, update):
         ''' Execute an update expression.  Should generally only be called implicitly.
         '''
-
+        
         self.flush()
         assert len(update.update_data) > 0
         collection = self.db[update.query.type.get_collection_name()]
@@ -218,6 +233,34 @@ class Session(object):
             index.ensure(collection)
         collection.update(update.query.query, update.update_data, upsert=update.get_upsert(), multi=update.get_multi())
     
+    def execute_find_and_modify(self, fm_exp):
+        self.flush()
+        # assert len(fm_exp.update_data) > 0
+        collection = self.db[fm_exp.query.type.get_collection_name()]
+        for index in fm_exp.query.type.get_indexes():
+            index.ensure(collection)
+        kwargs = {
+            'query' : fm_exp.query.query, 
+            'update' : fm_exp.update_data, 
+            'upsert' : fm_exp.get_upsert(), 
+        }
+        
+        if fm_exp.query.get_fields():
+            kwargs['fields'] = {}
+            for f in fm_exp.query.get_fields():
+                kwargs['fields'][str(f)] = True
+        if fm_exp.query.sort:
+            kwargs['sort'] = fm_exp.query.sort
+        if fm_exp.get_new():
+            kwargs['new'] = fm_exp.get_new()
+        if fm_exp.get_remove():
+            kwargs['remove'] = fm_exp.get_remove()
+        
+        value = collection.find_and_modify(**kwargs)        
+        if kwargs['upsert'] and not kwargs.get('new') and len(value) == 0:
+            return value
+        
+        return fm_exp.query.type.unwrap(value, fields=fm_exp.query.get_fields())
     
     def get_indexes(self, cls):
         ''' Get the index information for the collection associated with 
@@ -236,10 +279,12 @@ class Session(object):
         for c in classes:
             self.db[c.get_collection_name()].remove()
     
-    def flush(self, safe=True):
+    def flush(self, safe=None):
         ''' Perform all database operations currently in the queue'''
+        if safe is None:
+            safe = self.safe
         for index, item in enumerate(self.queue):
-            item.commit(self.db)
+            item.commit(self.db, safe=safe)
         self.clear()
             
     
