@@ -25,7 +25,7 @@ from pymongo import ASCENDING, DESCENDING
 from copy import copy, deepcopy
 
 from mongoalchemy.fields import BadValueException
-from mongoalchemy.query_expression import QueryExpression, BadQueryException
+from mongoalchemy.query_expression import QueryExpression, BadQueryException, flatten
 
 class UpdateExpression(object):
     def __init__(self, query):
@@ -103,6 +103,8 @@ class UpdateExpression(object):
         
     def remove(self, qfield, value):
         ''' Atomically remove ``value`` from ``qfield``'''
+        if isinstance(value, QueryExpression):
+            return self._atomic_expression_op('$pull', qfield, value)
         return self._atomic_list_op('$pull', qfield, value)
         
     def remove_all(self, qfield, *value):
@@ -150,6 +152,16 @@ class UpdateExpression(object):
         self.update_data[op][qfield.get_absolute_name()] = qfield.child_type().wrap(value)
         return self
     
+    def _atomic_expression_op(self, op, qfield, value):
+        qfield = self.query.resolve_name(qfield)
+        if op not in qfield.valid_modifiers:
+            raise InvalidModifierException(qfield, op)
+        
+        if op not in self.update_data:
+            self.update_data[op] = {}
+        self.update_data[op][qfield.get_absolute_name()] = flatten(value.obj)
+        return self
+    
     def _atomic_op(self, op, qfield, value):
         qfield = self.query.resolve_name(qfield)
         if op not in qfield.valid_modifiers:
@@ -180,6 +192,23 @@ class UpdateExpression(object):
         ''' Execute the update expression on the database '''
         self.session.execute_update(self)
 
+class FindAndModifyExpression(UpdateExpression):
+    def __init__(self, query, new, remove):
+        self.__new = new
+        self.__remove = remove
+        UpdateExpression.__init__(self, query)
+    
+    def get_remove(self):
+        return self.__remove
+    
+    def get_new(self):
+        return self.__new
+    
+    def execute(self):
+        ''' Execute the find and modify expression on the database '''
+        return self.session.execute_find_and_modify(self)
+        
+
 class UpdateException(Exception):
     ''' Base class for exceptions related to updates '''
     pass
