@@ -29,6 +29,30 @@ def update_test_setup():
     s.clear_collection(T)
     return s.query(T)
 
+# Find and Modify
+def test_find_and_modify():
+    s = get_session()
+    s.clear_collection(T, T2)
+    # insert
+    value = s.query(T).filter_by(i=12341).find_and_modify().set(i=12341).upsert().execute()
+    assert value == {}
+    assert s.query(T).one().i == 12341
+    # update
+    value = s.query(T).filter_by(i=12341).ascending(T.i).find_and_modify().set(i=9999).execute()
+    assert value.i == 12341
+    assert s.query(T).one().i == 9999
+    
+    # return new
+    value = s.query(T).filter_by(i=9999).fields(T.i).find_and_modify(new=True).set(i=8888).execute()
+    assert value.i == 8888, value.i
+    assert s.query(T).one().i == 8888
+
+    # remove
+    value = s.query(T).filter_by(i=8888).find_and_modify(remove=True).execute()
+    assert value.i == 8888, value.i
+    assert s.query(T).first() is None
+
+
 # General Update tests
 
 def test_multi():
@@ -96,6 +120,44 @@ def test_remove_obj():
     assert s.query(T).count() == 0
     t2 = T(i=3)
     s.remove(t2)
+
+def test_remove_expression():
+    class Page(Document):
+        text = StringField()
+        page_num = IntField()
+
+    class Book(Document):
+        isbn = StringField()
+        pages = ListField(DocumentField(Page))
+    
+    session = get_session()
+    session.clear_collection(Book)
+
+    ISBN = '3462784290890'
+    book = Book(isbn=ISBN, pages=[])
+    for i in range(0,10):
+        book.pages.append(Page(text='p%d' % i, page_num = i))
+
+    session.insert(book)
+
+    query = session.query(Book).filter(Book.isbn == ISBN, Book.pages.page_num > 5)
+    query.remove(Book.pages, Page.page_num == 8).execute()
+    book = session.query(Book).filter(Book.isbn == ISBN).one()
+    assert len(book.pages) == 9
+
+@raises(InvalidModifierException)
+def test_remove_expression_error():
+    class Page(Document):
+        text = StringField()
+        page_num = IntField()
+    
+    class Book(Document):
+        isbn = StringField()
+        pages = ListField(DocumentField(Page))
+    session = get_session()
+    session.query(Book).remove(Book.isbn, Page.page_num == 8)
+
+
 # SET
 
 def set_test():
